@@ -1,12 +1,12 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "separedit" "0.3.37"
+(define-package "separedit" "20250506.833"
   "Edit comment/string/docstring/code block in separate buffer."
   '((emacs         "25.1")
     (dash          "2.18")
-    (edit-indirect "0.1.5"))
+    (edit-indirect "0.1.11"))
   :url "https://github.com/twlz0ne/separedit.el"
-  :commit "91a41ff8044e7cbeb82159f6c595393606e5e744"
-  :revdesc "91a41ff8044e"
+  :commit "5cb46a65fc6e12b753dce8f581fbfa144d011a80"
+  :revdesc "5cb46a65fc6e"
   :keywords '("tools" "languages" "docs")
   :authors '(("Gong Qijian" . "gongqijian@gmail.com"))
   :maintainers '(("Gong Qijian" . "gongqijian@gmail.com")))
