@@ -4,11 +4,11 @@
 
 ;; Author: Gong Qijian <gongqijian@gmail.com>
 ;; Created: 2019/04/06
-;; Package-Version: 0.3.37
-;; Package-Revision: 91a41ff8044e
-;; Last-Updated: 2022-11-17 14:42:59 +0800
+;; Package-Version: 20250506.833
+;; Package-Revision: 5cb46a65fc6e
+;; Last-Updated: 2024-10-12 09:24:37 +0800
 ;;           by: Gong Qijian
-;; Package-Requires: ((emacs "25.1") (dash "2.18") (edit-indirect "0.1.5"))
+;; Package-Requires: ((emacs "25.1") (dash "2.18") (edit-indirect "0.1.11"))
 ;; URL: https://github.com/twlz0ne/separedit.el
 ;; Keywords: tools languages docs
 
@@ -398,6 +398,26 @@ Taken from `markdown-code-lang-modes'."
   :group 'separedit
   :type 'list)
 
+(defcustom separedit-major-mode-remap-alist
+  '((c-mode             . c-ts-mode)
+    (c++-mode           . c++-ts-mode)
+    (csharp-mode        . csharp-ts-mode)
+    (css-mode           . css-ts-mode)
+    (elixir-mode        . elixir-ts-mode)
+    (go-mode            . go-ts-mode)
+    (java-mode          . java-ts-mode)
+    (js-mode            . js-ts-mode)
+    (lua-mode           . lua-ts-mode)
+    (nix-mode           . nix-ts-mode)
+    (python-mode        . python-ts-mode)
+    (ruby-mode          . ruby-ts-mode)
+    (rust-mode          . rust-ts-mode)
+    (sh-mode            . bash-ts-mode)
+    (typescript-mode    . typescript-ts-mode))
+  "Alist to override `major-mode-remap-alist` in separedit."
+  :group 'separedit
+  :type 'alist)
+
 (defcustom separedit-comment-delimiter-alist
   '((("//+!\\(?:<\\)?" "//+\\(?:<\\)?" "\\*+") . (c-mode
                                                   c++-mode
@@ -421,7 +441,8 @@ Taken from `markdown-code-lang-modes'."
                           racket-mode
                           scheme-mode
                           fennel-mode))
-    (("#+")            . (conf-mode nix-mode python-mode ruby-mode yaml-mode)))
+    (("#+")            . ( conf-mode elixr-mode nix-mode python-mode ruby-mode
+                           yaml-mode)))
   "Alist of comment delimiter regexp.
 
 Each element should be in one of the following forms:
@@ -685,6 +706,17 @@ Return nil if reached the end of the buffer."
              (symbol-function mode))
         mode)))
 
+(defun separedit--assoc-fallback (mode alist assocfn)
+  "Call associate function ASSOCFN on MODE and ALIST."
+  (or (funcall assocfn mode alist)
+      (funcall assocfn (separedit--get-real-mode mode) alist)
+      (funcall assocfn (get mode 'derived-mode-parent) alist)
+      (funcall assocfn
+               (or (car (rassq mode separedit-major-mode-remap-alist))
+                   (and (boundp 'major-mode-remap-alist)
+                        (car (rassq mode major-mode-remap-alist))))
+               alist)))
+
 (defun separedit--get-mode-comment (mode symbol)
   "Get comment of MODE by SYMBOL."
   (or (get mode symbol)
@@ -702,6 +734,7 @@ Return nil if reached the end of the buffer."
 
 (defcustom separedit-string-quotes-alist
   '((python-mode     . ("\"\"\"" "'''" "\"" "'"))
+    (elixir-mode     . ("\"\"\"" "\"" "'"))
     (js-mode         . ("\"" "'" "`"))
     (lua-mode        . ("\"" "'" (or (seq "[" (zero-or-more "=") "[")
                                      (seq "]" (zero-or-more "=") "]"))))
@@ -725,11 +758,26 @@ Each item may be one of the following forms:
 (defun separedit--point-at-string (&optional pos)
   "Determine if point POS at string or not."
   (or (nth 3 (syntax-ppss pos))
-      (and (derived-mode-p 'python-mode)
+      (and (derived-mode-p 'nix-ts-mode 'python-mode 'python-ts-mode)
            (memq (face-at-point) '(font-lock-string-face font-lock-doc-face))
            (unless (bobp)
              (memq (get-char-property (1- (point)) 'face)
                    '(font-lock-string-face font-lock-doc-face))))
+      (and (derived-mode-p 'elixir-ts-mode)
+           (if (region-active-p)
+               ;; Seems the `elisir-ts-mode' will refresh whole region when
+               ;; moving point:
+               ;; (list (face-at-point) (progn (backward-char) (face-at-point)))
+               ;; => (font-lock-doc-face region)
+               ;; => (font-lock-doc-face font-lock-doc-face) ;; expected
+               ;; Version          : 1.2
+               ;; Package-Version: 20230321.1458
+               ;; Package-Commit: 0d4ef4794655a2a3c5324e07eef46dc4766ad65d
+               (memq (face-at-point) '(font-lock-doc-face region))
+             (memq (face-at-point) '(font-lock-doc-face)))
+           (not (or (looking-back "^\s+\"\\{3\\}")               ;; end of doc
+                    (and (looking-back "@\\(?:module\\)?doc\s+") ;; beg of doc
+                         (looking-at "\"\\{3\\}$")))))
       (and (derived-mode-p 'perl-mode)
            (memq (face-at-point) '(perl-heredoc))
            (not (or (separedit--string-quotes pos)
@@ -849,11 +897,8 @@ for example:
 If MODE is nil, it defaults to `major-mode'.
 If there is no comment delimiter regex for MODE, return `comment-start-skip'."
   (let* ((mode (or mode major-mode))
-         (def (or (separedit--rassoc mode separedit-comment-delimiter-alist)
-                  (separedit--rassoc (get mode 'derived-mode-parent)
-                                     separedit-comment-delimiter-alist)
-                  (separedit--rassoc (separedit--get-real-mode mode)
-                                     separedit-comment-delimiter-alist))))
+         (def (separedit--assoc-fallback
+               mode separedit-comment-delimiter-alist 'separedit--rassoc)))
     (pcase (car def)
       ((and fn (pred functionp)) (funcall fn))
       ((and strs (pred consp)) (concat "^[\s\t]*\\(?:"
@@ -896,7 +941,7 @@ If there is no comment delimiter regex for MODE, return `comment-start-skip'."
 (defun separedit--point-at-comment (&optional point)
   "Return non-nil (face or t) if POINT at comment."
   (let ((face (unless (separedit--point-at-string)
-                (get-text-property (or point (if (and (eobp) (not (bolp)))
+                (get-text-property (or point (if (and (eolp) (not (bolp)))
                                                  (1- (point))
                                                (point)))
                                    'face))))
@@ -1082,11 +1127,8 @@ If MODE is nil, use ‘major-mode’."
 (defun separedit--get-comment-encloser (&optional mode)
   "Return comment ‘(begin-encloser end-encloser)’ for MODE."
   (let* ((mode (or mode major-mode))
-         (def (or (separedit--rassoc mode separedit-comment-encloser-alist)
-                  (separedit--rassoc (get mode 'derived-mode-parent)
-                                     separedit-comment-encloser-alist)
-                  (separedit--rassoc (separedit--get-real-mode mode)
-                                     separedit-comment-encloser-alist))))
+         (def (separedit--assoc-fallback
+               mode separedit-comment-encloser-alist 'separedit--rassoc)))
     (if def
         (if (functionp (car def))
             (funcall (car def))
@@ -1241,7 +1283,8 @@ LANG is a string, and the returned major mode is a symbol."
 
 (defun separedit-get-mode-quotes (mode)
   "Return a list of quote string for MODE."
-  (let ((aval (assoc-default mode separedit-string-quotes-alist)))
+  (let ((aval (separedit--assoc-fallback
+               mode separedit-string-quotes-alist 'assoc-default)))
     (cond ((symbolp aval) (assoc-default (or aval t) separedit-string-quotes-alist))
           (t aval))))
 
@@ -1252,6 +1295,25 @@ LANG is a string, and the returned major mode is a symbol."
     (when (looking-back regexp (line-beginning-position))
       (match-string-no-properties 1))))
 
+(cl-defgeneric separedit--indent-of-straight (_beg _end)
+  "Return the base indent of straight block (e.g. Info-mode) betwen BEG and END."
+  nil)
+
+(cl-defmethod separedit--indent-of-straight (beg end &context (major-mode
+                                                               Info-mode))
+
+  (save-excursion
+    (goto-char beg)
+    (let (indent)
+      (while (< (point) end)
+        (unless (looking-at-p "[ \t]*$")
+          (back-to-indentation)
+          (setq indent (if indent
+                           (min indent (current-column))
+                         (current-column))))
+        (forward-line))
+      indent)))
+
 (cl-defgeneric separedit--indent-of-string-block-0
     (_quotes _beg _end _str-start _beg-at-newline _end-at-newline)
   "Called by `separedit--indent-of-string-block' in final clause of cond."
@@ -1302,7 +1364,8 @@ Return value is in the form of (indent-length indent-line1)."
              (goto-char end)
              (cons
               (+ (current-column)
-                 (or (cdr (assoc major-mode separedit-string-indent-offset-alist))
+                 (or (let ((rmod (car (rassq major-mode separedit-major-mode-remap-alist))))
+                       (cdr (assoc (or rmod major-mode) separedit-string-indent-offset-alist)))
                      0))
               t))
             ((not (string= str-start ""))
@@ -1421,7 +1484,24 @@ Block info example:
               (goto-char (car comment-or-string-region))
               (when (and (bolp) (not (bobp)))
                 (backward-char))
-              (separedit-looking-back-heredoc-language))))
+              (let ((eof-mark (separedit-looking-back-heredoc-language)))
+                ;; Exclude the eof-mark from `comment-or-string-region',
+                ;; for example in 27.2 or older:
+                ;;
+                ;;     print <<'EOF';
+                ;;     PERL|
+                ;;     EOF
+                ;;
+                ;; Expected region (15 22), but actual (15 26).
+                (when eof-mark
+                  (save-excursion
+                    (goto-char (cadr comment-or-string-region))
+                    (when (re-search-backward
+                           eof-mark (car comment-or-string-region) t)
+                      (setq comment-or-string-region
+                            (list (car comment-or-string-region)
+                                  (1- (point)))))))
+                eof-mark))))
          (heredoc-mode (if heredoc-lang (separedit-get-lang-mode heredoc-lang)))
          (indent-line1 nil)
          (string-indent
@@ -1619,7 +1699,7 @@ It will override by the key that `separedit' binding in source buffer.")
                (scp (nth 3 separedit--help-variable-edit-info))
                (buf (nth 4 separedit--help-variable-edit-info))
                (val (if (nth 2 separedit--help-variable-edit-info)
-                        (substring-no-properties (buffer-string)) 
+                        (substring-no-properties (buffer-string))
                       (list 'quote (car (read-from-string (buffer-string)))))))
           (cond
            ((and (eq scp 'local) buf) (with-current-buffer buf (eval `(setq-local ,sym ,val))))
@@ -1641,15 +1721,15 @@ It will override by the key that `separedit' binding in source buffer.")
            ;; Clone a buffer to protect the folding of text in edit buffer.
            (clone-buffer (concat (buffer-name) " <clone>")))
           (function-backup
-           ;; Temprary disable the ‘edit-indirect--clean-up’ (but who calls
+           ;; Temprary disable the ‘edit-indirect--abort’ (but who calls
            ;; this function after the clone buffer is killed?)
-           (symbol-function 'edit-indirect--clean-up)))
+           (symbol-function 'edit-indirect--abort)))
       (unwind-protect
           (with-current-buffer edit-buffer-clone
-            (fset 'edit-indirect--clean-up (lambda ()))
+            (fset 'edit-indirect--abort #'ignore)
             (separedit--apply-changes))
         (kill-buffer edit-buffer-clone)
-        (fset 'edit-indirect--clean-up function-backup)
+        (fset 'edit-indirect--abort function-backup)
         (if (and separedit-write-file-when-execute-save
                  (buffer-file-name source-buffer))
             (with-current-buffer source-buffer
@@ -1669,7 +1749,7 @@ It will override by the key that `separedit' binding in source buffer.")
         (mark-beg (overlay-start edit-indirect--overlay))
         (mark-end (overlay-end edit-indirect--overlay)))
     (separedit--apply-changes)
-    (edit-indirect--clean-up) ;; Returned to source buffer
+    (edit-indirect--abort t) ;; Returned to source buffer
     (goto-char
      (save-excursion
        (save-restriction
@@ -1786,7 +1866,7 @@ MAX-WIDTH       maximum width that can be removed"
 (defun separedit--remove-c/c++-macro-delimiter (_ &optional _)
   "Remove c/c++ macro delimiter of each line when etering edit buffer."
   (let ((inhibit-read-only t)
-        ;; Regexp from `separedit-block-regexp-plists' is not working as 
+        ;; Regexp from `separedit-block-regexp-plists' is not working as
         ;; expected when search backward, use the following instead:
         (regexp "\\(?:[^\s\t]\\|^[\s\t]\\)\\([\s\t]*\\\\[\s\t]*\\)"))
     (save-excursion
@@ -1972,8 +2052,8 @@ If you just want to check `major-mode', use `derived-mode-p'."
                                        separedit-single-quote-string-mode
                                        separedit-double-quote-string-mode)))))))
                (complete-with-action action obarray string pred))))
-   #'commandp t nil 'separedit--mode-history (or (car separedit--mode-history)
-                                                 (format "%s" major-mode))))
+   #'commandp nil nil 'separedit--mode-history (or (car separedit--mode-history)
+                                                   (format "%s" major-mode))))
 
 (defvar separedit-mode-map (make-sparse-keymap) "Keymap used in comment edit buffer.")
 
@@ -2053,6 +2133,7 @@ If you just want to check `major-mode', use `derived-mode-p'."
 
 ;;; vterm
 
+(defvar vterm-shell)
 (declare-function vterm-delete-region "vterm")
 (declare-function vterm-insert "vterm")
 (declare-function vterm-copy-mode "vterm")
@@ -2078,6 +2159,10 @@ This function is used to instead of ‘replace-match’."
   "Edit content after vterm prompt."
   (interactive)
   (let* ((edit-indirect-after-creation-hook #'separedit--buffer-creation-setup)
+         (lang-mode
+          (let ((sh (file-name-nondirectory vterm-shell)))
+            (cond ((member sh '("bash" "zsh" "ksh" "csh")) 'sh-mode)
+                  ((member sh '("fish")) 'fish-mode))))
          (start-point (vterm--get-prompt-point))
          (end-point (save-excursion
                       (goto-char (point-max))
@@ -2085,6 +2170,8 @@ This function is used to instead of ‘replace-match’."
                       (max start-point (1+ (point)))))
          (edit-indirect-guess-mode-function
           (lambda (_buffer _beg _end)
+            (when lang-mode
+              (funcall lang-mode))
             (setq-local separedit--inhibit-read-only t)
             (setq-local edit-indirect--inhibit-read-only t)
             (setq-local separedit-replace-match-function
@@ -2205,24 +2292,33 @@ but users can also manually select it by pressing `C-u \\[separedit]'."
     (separedit-dwim-described-variable))
    ((memq major-mode '(vterm-mode))
     (separedit-dwim-vterm))
-   (t (separedit-dwim-default
-       (or block
-           ;; minibuffer
-           (when (and (minibufferp (current-buffer))
-                      (not (separedit--point-at-string)))
-             (list :beginning (+ (point-min) (length (minibuffer-prompt)))
-                   :end       (point-max)
-                   :lang-mode 'emacs-lisp-mode))
-           ;; region
-           (when (region-active-p)
-             (list :beginning (region-beginning)
-                   :end (if (and (= ?\n (char-before (region-end)))
-                                 (not (= ?\n (char-after (region-end)))))
-                            (1- (region-end))
-                          (region-end))
-                   :lang-mode (if (called-interactively-p 'any)
-                                  (intern (separedit--select-mode))
-                                separedit-default-mode))))))))
+   (t (let ((current-prefix-arg (if (region-active-p) '(4) current-prefix-arg)))
+        (separedit-dwim-default
+         (or block
+             (let ((strp (separedit--point-at-string)))
+               (or
+                ;; minibuffer
+                (when (and (minibufferp (current-buffer)) (not strp))
+                  (list :beginning (+ (point-min) (length (minibuffer-prompt)))
+                        :end       (point-max)
+                        :lang-mode 'emacs-lisp-mode))
+                ;; region
+                (when (region-active-p)
+                  (let ((block (if strp
+                                   (separedit--block-info)
+                                 (when-let ((indent (separedit--indent-of-straight
+                                                     (region-beginning)
+                                                     (region-end))))
+                                   (list :indent-length indent
+                                         :regexps (list :straight t))))))
+                    (plist-put
+                     (plist-put block :beginning (region-beginning))
+                     :end (if (and (= ?\n (char-before (region-end)))
+                                   (if (char-after (region-end))
+                                       (not (= ?\n (char-after (region-end))))
+                                     t))
+                              (1- (region-end))
+                            (region-end)))))))))))))
 
 ;;;###autoload
 (defalias 'separedit 'separedit-dwim)
